@@ -1,10 +1,10 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
- * 
+ *
  * make-iso.c: code to generate iso files
  *
  * Copyright (C) 2002-2004 Red Hat, Inc.
  * Copyright (C) 2005-2006 William Jon McCann <mccann@jhu.edu>
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -77,7 +77,6 @@ struct NautilusBurnIsoPrivate {
 	gboolean             debug;
 };
 
-
 /* Signals */
 enum {
 	PROGRESS_CHANGED,
@@ -153,7 +152,7 @@ write_all (int   fd,
 {
 	int bytes;
 	int res;
-	
+
 	bytes = 0;
 	while (bytes < len) {
 		res = write (fd, buf + bytes, len - bytes);
@@ -177,7 +176,7 @@ copy_file (const char *source,
 	if (link (source, dest) == 0) {
 		return;
 	}
-	
+
 	if (g_stat (source, &stat_buf) != 0) {
 		g_warning ("Trying to copy nonexisting file\n");
 		return;
@@ -188,7 +187,7 @@ copy_file (const char *source,
 		g_warning ("Can't copy file (open source failed)\n");
 		return;
 	}
-	
+
 	dfd = g_open (dest, O_WRONLY | O_CREAT, stat_buf.st_mode);
 	if (dfd == -1) {
 		close (sfd);
@@ -210,7 +209,7 @@ escape_path (const char *str)
 	char       *escaped, *d;
 	const char *s;
 	int         len;
-	
+
 	s = str;
 	len = 1;
 	while (*s != 0) {
@@ -218,13 +217,13 @@ escape_path (const char *str)
 		    *s == '=') {
 			len++;
 		}
-		
+
 		len++;
 		s++;
 	}
-	
+
 	escaped = g_malloc (len);
-	
+
 	s = str;
 	d = escaped;
 	while (*s != 0) {
@@ -232,11 +231,11 @@ escape_path (const char *str)
 		    *s == '=') {
 			*d++ = '\\';
 		}
-		
+
 		*d++ = *s++;
 	}
 	*d = 0;
-	
+
 	return escaped;
 }
 
@@ -248,11 +247,11 @@ dir_is_empty (const char *virtual_path)
 	GnomeVFSResult           result;
 	char                    *escaped_path, *uri;
 	gboolean                 found_file;
-	
+
 	escaped_path = gnome_vfs_escape_path_string (virtual_path);
 	uri = g_strconcat ("burn:///", escaped_path, NULL);
 	g_free (escaped_path);
-	
+
 	result = gnome_vfs_directory_open (&handle, uri, GNOME_VFS_FILE_INFO_DEFAULT);
 	g_free (uri);
 
@@ -263,7 +262,7 @@ dir_is_empty (const char *virtual_path)
 	info = gnome_vfs_file_info_new ();
 
 	found_file = FALSE;
-	
+
 	while (TRUE) {
 		result = gnome_vfs_directory_read_next (handle, info);
 		if (result != GNOME_VFS_OK)
@@ -276,7 +275,7 @@ dir_is_empty (const char *virtual_path)
 			gnome_vfs_file_info_clear (info);
 			continue;
 		}
-		
+
 		found_file = TRUE;
 		break;
 	}
@@ -327,6 +326,7 @@ ask_disable_joliet (GtkWindow *parent)
 					 GTK_BUTTONS_OK_CANCEL,
 					 _("Disable Microsoft Windows compatibility?"));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  "%s",
 						  _("Some files don't have a suitable name for a Windows-compatible CD.\nDo you want to continue with Windows compatibility disabled?"));
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Windows compatibility"));
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -349,7 +349,7 @@ static void
 graft_file_end_dir_visitor (struct graft_state *state)
 {
 	char *last_slash;
-	
+
 	if (state->copy_depth > 0) {
 		state->copy_depth--;
 		if (state->copy_depth == 0) {
@@ -420,7 +420,7 @@ graft_file_visitor (const char           *rel_path,
 		}
 		return TRUE;
 	}
-	
+
 	if (info->type != GNOME_VFS_FILE_TYPE_DIRECTORY) {
 		mapping = get_backing_file (rel_path);
 		if (mapping != NULL) {
@@ -458,12 +458,12 @@ graft_file_visitor (const char           *rel_path,
 				g_warning ("Couldn't create temp subdir\n");
 				*recurse = FALSE;
 			}
-		} 
+		}
 	}
 
 	return TRUE;
 }
-	
+
 static void
 create_graft_file (GnomeVFSURI          *uri,
 		   const char           *prefix,
@@ -509,7 +509,7 @@ create_graft_file (GnomeVFSURI          *uri,
 
 		recurse = FALSE;
 		stop = ! graft_file_visitor (rel_path, info, state, &recurse);
-		
+
 		if (! stop
 		    && recurse
 		    && info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
@@ -560,13 +560,13 @@ nautilus_burn_iso_graft_free (NautilusBurnIsoGraft *graft,
 	g_ptr_array_free (graft->array, TRUE);
 
 	for (l = graft->remove_files; l != NULL; l = l->next) {
-		if (cleanup) {
+		if (cleanup && l->data != NULL) {
 			g_remove ((char *)l->data);
 		}
 		g_free (l->data);
 	}
 	g_list_free (graft->remove_files);
-	
+
 	g_free (graft);
 	graft = NULL;
 }
@@ -605,7 +605,7 @@ create_temp_dir (void)
 	template_dir = g_build_filename (g_get_tmp_dir (), template, NULL);
 	g_free (template);
 	temp_dir = mkdtemp (template_dir);
-	
+
 	if (temp_dir == NULL) {
 		g_free (template_dir);
 	}
@@ -649,7 +649,7 @@ nautilus_burn_iso_graft_new (const char *uri_string)
 	if (state.tmpdir == NULL) {
 		goto done;
 	}
-	
+
 	state.emptydir = g_build_filename (state.tmpdir, "emptydir", NULL);
 	res = g_mkdir (state.emptydir, 0777);
 	if (res != 0) {
@@ -687,11 +687,9 @@ process_error (NautilusBurnIso *iso,
 		       nautilus_burn_iso_table_signals [PROGRESS_CHANGED], 0,
 		       1.0, (long)-1);
 
-	if (iso->priv->process->last_error == NULL) {
-		iso->priv->process->last_error = g_strdup (message);
-	}
-
-	iso->priv->process->result = NAUTILUS_BURN_ISO_RESULT_ERROR;
+	nautilus_burn_process_set_error (iso->priv->process,
+					 message,
+					 NAUTILUS_BURN_ISO_RESULT_ERROR);
 }
 
 /* Originally eel_make_valid_utf8 */
@@ -906,9 +904,11 @@ nautilus_burn_iso_run_process (NautilusBurnIso            *iso,
 	int                  result;
 	GError              *local_error = NULL;
 	NautilusBurnProcess *process;
+	char                *error_message;
+	int                  error_code;
 
  retry:
-	
+
 	process = nautilus_burn_process_new ();
 	process->debug = iso->priv->debug;
 
@@ -936,11 +936,18 @@ nautilus_burn_iso_run_process (NautilusBurnIso            *iso,
 		goto retry;
 	}
 
-	if (process->last_error) {
+	if (local_error != NULL) {
 		g_set_error (error,
 			     NAUTILUS_BURN_ISO_ERROR,
 			     NAUTILUS_BURN_ISO_ERROR_GENERAL,
-			     process->last_error);
+			     local_error->message);
+		g_error_free (local_error);
+	} else if (nautilus_burn_process_get_error (process, &error_message, &error_code)) {
+		g_set_error (error,
+			     NAUTILUS_BURN_ISO_ERROR,
+			     NAUTILUS_BURN_ISO_ERROR_GENERAL,
+			     error_message);
+		g_free (error_message);
 	}
 
 	result = process->result;
@@ -1014,7 +1021,7 @@ nautilus_burn_iso_graft_get_info (NautilusBurnIsoGraft *graft,
 			   &stderr_data,
 			   &exit_status,
 			   &sub_error)) {
-		g_set_error (error, NAUTILUS_BURN_ISO_ERROR, 
+		g_set_error (error, NAUTILUS_BURN_ISO_ERROR,
 			     NAUTILUS_BURN_ISO_ERROR_GENERAL,
 			     _("Could not run sub process: %s."),
 			     sub_error->message);
@@ -1037,7 +1044,7 @@ nautilus_burn_iso_graft_get_info (NautilusBurnIsoGraft *graft,
 				goto retry;
 			} else {
 				g_set_error (error,
-					     NAUTILUS_BURN_ISO_ERROR, 
+					     NAUTILUS_BURN_ISO_ERROR,
 					     NAUTILUS_BURN_ISO_ERROR_GENERAL,
 					     _("The operation was cancelled by the user."));
 				result = FALSE;
@@ -1135,7 +1142,7 @@ mkisofs_stderr_line (NautilusBurnProcess   *process,
 		g_string_erase (filename, 0, strchr (filename->str, '(') - filename->str + 1);
 		/* Truncate down to the last ) */
 		g_string_truncate (filename, strrchr (filename->str, ')') - filename->str - 1);
-				
+
 		utf8_filename = ncb_make_valid_utf8 (filename->str);
 		msg = g_strdup_printf (_("Some files have invalid filenames: \n%s"), utf8_filename);
 		if (iso->priv->debug) {
@@ -1235,7 +1242,7 @@ nautilus_burn_iso_make_from_graft (NautilusBurnIso             *iso,
 								 size);
 
 			g_set_error (error,
-				     NAUTILUS_BURN_ISO_ERROR, 
+				     NAUTILUS_BURN_ISO_ERROR,
 				     NAUTILUS_BURN_ISO_ERROR_GENERAL,
 				     "%s", message);
 			g_free (message);
@@ -1315,7 +1322,7 @@ nautilus_burn_iso_make (NautilusBurnIso             *iso,
 	graft = nautilus_burn_iso_graft_new (BURN_URI);
 	if (graft == NULL || graft->array == NULL) {
 		g_set_error (error,
-			     NAUTILUS_BURN_ISO_ERROR, 
+			     NAUTILUS_BURN_ISO_ERROR,
 			     NAUTILUS_BURN_ISO_ERROR_GENERAL,
 			     _("There are no files to write to disc."));
 		result = NAUTILUS_BURN_ISO_RESULT_ERROR;
@@ -1363,7 +1370,7 @@ get_disc_info (NautilusBurnIso   *iso,
 							      &has_data,
 							      &_has_audio);
 	if (_has_audio) {
-		device_arg = g_strdup_printf ("%s", drive->cdrecord_id);
+		device_arg = g_strdup_printf ("%s", nautilus_burn_drive_get_device (drive));
 
 		argv = g_ptr_array_new ();
 		g_ptr_array_add (argv, "cdrdao");
@@ -1371,9 +1378,9 @@ get_disc_info (NautilusBurnIso   *iso,
 		g_ptr_array_add (argv, "--device");
 		g_ptr_array_add (argv, device_arg);
 		g_ptr_array_add (argv, NULL);
-		
+
 	} else {
-		device_arg = g_strdup_printf ("-dev=%s", drive->cdrecord_id);
+		device_arg = g_strdup_printf ("-dev=%s", nautilus_burn_drive_get_device (drive));
 
 		argv = g_ptr_array_new ();
 		g_ptr_array_add (argv, "readcd");
@@ -1394,7 +1401,7 @@ get_disc_info (NautilusBurnIso   *iso,
 			    &stderr_data,
 			    &exit_status,
 			    &sub_error)) {
-		g_set_error (error, NAUTILUS_BURN_ISO_ERROR, 
+		g_set_error (error, NAUTILUS_BURN_ISO_ERROR,
 			     NAUTILUS_BURN_ISO_ERROR_GENERAL,
 			     _("Could not run sub process: %s."),
 			     sub_error->message);
@@ -1484,6 +1491,7 @@ nautilus_burn_iso_make_from_drive (NautilusBurnIso             *iso,
 	iso->priv->debug = (flags & NAUTILUS_BURN_IMAGE_CREATE_DEBUG);
 	iso->priv->filename = g_strdup (filename);
 
+	sub_error = NULL;
 	if (! get_disc_info (iso, drive, &iso_size, &has_audio, &sub_error)) {
 		g_propagate_error (error, sub_error);
 		return NAUTILUS_BURN_ISO_RESULT_ERROR;
@@ -1515,7 +1523,7 @@ nautilus_burn_iso_make_from_drive (NautilusBurnIso             *iso,
 			message = get_image_space_error_message (iso_size,
 								 size);
 
-			g_set_error (error, NAUTILUS_BURN_ISO_ERROR, 
+			g_set_error (error, NAUTILUS_BURN_ISO_ERROR,
 				     NAUTILUS_BURN_ISO_ERROR_GENERAL,
 				     "%s", message);
 			g_free (message);
@@ -1542,7 +1550,7 @@ nautilus_burn_iso_make_from_drive (NautilusBurnIso             *iso,
 		g_ptr_array_add (argv, "--datafile");
 		g_ptr_array_add (argv, (char *)filename);
 		g_ptr_array_add (argv, "--device");
-		g_ptr_array_add (argv, (char *)drive->cdrecord_id);
+		g_ptr_array_add (argv, (char *)nautilus_burn_drive_get_device (drive));
 		g_ptr_array_add (argv, "-v");
 		g_ptr_array_add (argv, "2");
 		g_ptr_array_add (argv, (char *)toc_filename_arg);
@@ -1554,7 +1562,7 @@ nautilus_burn_iso_make_from_drive (NautilusBurnIso             *iso,
 
 		filename_arg = g_strdup_printf ("f=%s", filename);
 
-		dev_arg = g_strdup_printf ("dev=%s", drive->cdrecord_id);
+		dev_arg = g_strdup_printf ("dev=%s", nautilus_burn_drive_get_device (drive));
 
 		argv = g_ptr_array_new ();
 		g_ptr_array_add (argv, "readcd");
